@@ -2,42 +2,59 @@
 # coding=utf-8
 # ------------------------------------------------------------------------
 
-import warnings
+import logging
+import sys
+import traceback
 
 from django import template
 from django.conf import settings
-from django.db.models import Q
 from django.http import HttpRequest
 
-from feincms.module.page.models import Page, PageManager
-from feincms.utils.templatetags import *
-from feincms.utils.templatetags import _parse_args
+from feincms.module.page.models import Page
+from feincms.utils.templatetags import (SimpleNodeWithVarAndArgs,
+    do_simple_node_with_var_and_args_helper,
+    SimpleAssignmentNodeWithVarAndArgs,
+    do_simple_assignment_node_with_var_and_args_helper)
+
+
+logger = logging.getLogger('feincms.templatetags.page')
 
 register = template.Library()
 
 
+# ------------------------------------------------------------------------
+# TODO: Belongs in some utility module
+def format_exception(e):
+    top = traceback.extract_tb(sys.exc_info()[2])[-1]
+    return u"'%s' in %s line %d" % (e, top[0], top[1])
+
+
 # ------------------------------------------------------------------------
 @register.assignment_tag(takes_context=True)
 def feincms_nav(context, feincms_page, level=1, depth=1):
     """
-    New, simplified version of the ``{% feincms_navigation %}`` template tag.
-
-    Generally we don't like abbreviations too much, but the similarity with
-    the HTML5 <nav> tag was just too tempting.
+    Saves a list of pages into the given context variable.
     """
 
     if isinstance(feincms_page, HttpRequest):
-        feincms_page = Page.objects.for_request(feincms_page, best_match=True)
+        try:
+            # warning: explicit Page reference here
+            feincms_page = Page.objects.for_request(
+                feincms_page, best_match=True)
+        except Page.DoesNotExist:
+            return []
 
     mptt_opts = feincms_page._mptt_meta
 
     # mptt starts counting at zero
     mptt_level_range = [level - 1, level + depth - 1]
 
-    queryset = Page.objects.in_navigation().filter(**{
-        '%s__gte' % mptt_opts.level_attr: mptt_level_range[0],
-        '%s__lt' % mptt_opts.level_attr: mptt_level_range[1],
-        })
+    queryset = feincms_page.__class__._default_manager.in_navigation().filter(
+        **{
+            '%s__gte' % mptt_opts.level_attr: mptt_level_range[0],
+            '%s__lt' % mptt_opts.level_attr: mptt_level_range[1],
+        }
+    )
 
     page_level = getattr(feincms_page, mptt_opts.level_attr)
 
@@ -64,11 +81,12 @@ def feincms_nav(context, feincms_page, level=1, depth=1):
             queryset = Page.objects.none()
 
         if parent:
-            # Special case for navigation extensions
             if getattr(parent, 'navigation_extension', None):
+                # Special case for navigation extensions
                 return list(parent.extended_navigation(depth=depth,
-                                    request=context.get('request')))
+                    request=context.get('request')))
 
+            # Apply descendant filter
             queryset &= parent.get_descendants()
 
     if depth > 1:
@@ -79,98 +97,48 @@ def feincms_nav(context, feincms_page, level=1, depth=1):
             # Subset filtering; allow children of parent as well
             parents.add(parent.id)
 
-        def _filter(iterable):
+        def _parentactive_filter(iterable):
             for elem in iterable:
                 if elem.parent_id in parents:
                     yield elem
                 parents.add(elem.id)
 
-        queryset = _filter(queryset)
+        queryset = _parentactive_filter(queryset)
 
-    if any((ext in getattr(feincms_page, '_feincms_extensions', []) for ext
-            in ('navigation', 'feincms.module.page.extensions.navigation'))):
+    if hasattr(feincms_page, 'navigation_extension'):
         # Filter out children of nodes which have a navigation extension
-        extended_node_rght = [] # mptt node right value
-
-        def _filter(iterable):
+        def _navext_filter(iterable):
+            current_navextension_node = None
             for elem in iterable:
-                if extended_node_rght:
-                    if getattr(elem, mptt_opts.right_attr) < extended_node_rght[-1]:
-                        # Still inside some navigation extension
-                        continue
-                    else:
-                        extended_node_rght.pop()
+                # Eliminate all subitems of last processed nav extension
+                if current_navextension_node is not None and \
+                   current_navextension_node.is_ancestor_of(elem):
+                    continue
 
+                yield elem
                 if getattr(elem, 'navigation_extension', None):
-                    yield elem
-                    extended_node_rght.append(getattr(elem, mptt_opts.right_attr))
-
-                    for extended in elem.extended_navigation(depth=depth,
-                            request=context.get('request')):
-                        if getattr(extended, mptt_opts.level_attr, 0) < level + depth - 1:
-                            yield extended
-
+                    current_navextension_node = elem
+                    try:
+                        for extended in elem.extended_navigation(depth=depth, request=context.get('request')):
+                            # Only return items from the extended navigation which
+                            # are inside the requested level+depth values. The
+                            # "-1" accounts for the differences in MPTT and
+                            # navigation level counting
+                            this_level = getattr(extended, mptt_opts.level_attr, 0)
+                            if this_level < level + depth - 1:
+                                yield extended
+                    except Exception as e:
+                        logger.warn("feincms_nav caught exception in navigation extension for page %d: %s", current_navextension_node.id, format_exception(e))
                 else:
-                    yield elem
+                    current_navextension_node = None
 
-        queryset = _filter(queryset)
+        queryset = _navext_filter(queryset)
 
     # Return a list, not a generator so that it can be consumed
     # several times in a template.
     return list(queryset)
 
 
-# ------------------------------------------------------------------------
-class NavigationNode(SimpleAssignmentNodeWithVarAndArgs):
-    """
-    Return a list of pages to be used for the navigation
-
-    level: 1 = toplevel, 2 = sublevel, 3 = sub-sublevel
-    depth: 1 = only one level, 2 = subpages too
-    extended: run navigation extension on returned pages, not only on top-level node
-
-    If you set depth to something else than 1, you might want to look into
-    the tree_info template tag from the mptt_tags library.
-
-    Example::
-
-        {% feincms_navigation of feincms_page as sublevel level=2,depth=1 %}
-        {% for p in sublevel %}
-            <a href="{{ p.get_absolute_url }}">{{ p.title }}</a>
-        {% endfor %}
-    """
-
-    def what(self, instance, args):
-        warnings.warn('feincms_navigation and feincms_navigation_extended have'
-            ' been deprecated and will be removed in FeinCMS v1.8. Start using'
-            ' the new, shiny and bug-free feincms_nav today!',
-            DeprecationWarning, stacklevel=3)
-
-        return feincms_nav(
-            self.render_context,
-            instance,
-            level=int(args.get('level', 1)),
-            depth=int(args.get('depth', 1)),
-            )
-register.tag('feincms_navigation',
-    do_simple_assignment_node_with_var_and_args_helper(NavigationNode))
-
-# ------------------------------------------------------------------------
-class ExtendedNavigationNode(NavigationNode):
-    def render(self, context):
-        self.render_context = context
-        try:
-            instance = self.in_var.resolve(context)
-        except template.VariableDoesNotExist:
-            context[self.var_name] = []
-            return ''
-
-        context[self.var_name] = self.what(instance, _parse_args(self.args, context))
-
-        return ''
-register.tag('feincms_navigation_extended',
-    do_simple_assignment_node_with_var_and_args_helper(ExtendedNavigationNode))
-
 # ------------------------------------------------------------------------
 class ParentLinkNode(SimpleNodeWithVarAndArgs):
     """
@@ -191,6 +159,7 @@ class ParentLinkNode(SimpleNodeWithVarAndArgs):
             return '#'
 register.tag('feincms_parentlink', do_simple_node_with_var_and_args_helper(ParentLinkNode))
 
+
 # ------------------------------------------------------------------------
 class LanguageLinksNode(SimpleAssignmentNodeWithVarAndArgs):
     """
@@ -241,14 +210,15 @@ class LanguageLinksNode(SimpleAssignmentNodeWithVarAndArgs):
 
             # hardcoded paths... bleh
             if key in translations:
-                links.append((key, name, translations[key].get_absolute_url()+trailing_path))
+                links.append((key, name,
+                    translations[key].get_absolute_url() + trailing_path))
             elif not only_existing:
                 links.append((key, name, None))
 
         return links
 register.tag('feincms_languagelinks', do_simple_assignment_node_with_var_and_args_helper(LanguageLinksNode))
 
-# ------------------------------------------------------------------------
+
 # ------------------------------------------------------------------------
 def _translate_page_into(page, language, default=None):
     """
@@ -270,6 +240,7 @@ def _translate_page_into(page, language, default=None):
         return default(page=page)
     return default
 
+
 # ------------------------------------------------------------------------
 class TranslatedPageNode(SimpleAssignmentNodeWithVarAndArgs):
     """
@@ -304,20 +275,23 @@ class TranslatedPageNode(SimpleAssignmentNodeWithVarAndArgs):
         return _translate_page_into(page, language, default=default)
 register.tag('feincms_translatedpage', do_simple_assignment_node_with_var_and_args_helper(TranslatedPageNode))
 
+
 # ------------------------------------------------------------------------
 class TranslatedPageNodeOrBase(TranslatedPageNode):
     def what(self, page, args):
         return super(TranslatedPageNodeOrBase, self).what(page, args, default=getattr(page, 'get_original_translation', page))
 register.tag('feincms_translatedpage_or_base', do_simple_assignment_node_with_var_and_args_helper(TranslatedPageNodeOrBase))
 
+
 # ------------------------------------------------------------------------
 @register.filter
 def feincms_translated_or_base(pages, language=None):
     if not hasattr(pages, '__iter__'):
-        pages = [ pages ]
+        pages = [pages]
     for page in pages:
         yield _translate_page_into(page, language, default=page.get_original_translation)
 
+
 # ------------------------------------------------------------------------
 @register.inclusion_tag("breadcrumbs.html")
 def feincms_breadcrumbs(page, include_self=True):
@@ -344,10 +318,12 @@ def feincms_breadcrumbs(page, include_self=True):
 
     return {"trail": bc}
 
+
 # ------------------------------------------------------------------------
 def _is_parent_of(page1, page2):
     return page1.tree_id == page2.tree_id and page1.lft < page2.lft and page1.rght > page2.rght
 
+
 @register.filter
 def is_parent_of(page1, page2):
     """
@@ -363,10 +339,12 @@ def is_parent_of(page1, page2):
     except AttributeError:
         return False
 
+
 # ------------------------------------------------------------------------
 def _is_equal_or_parent_of(page1, page2):
     return page1.tree_id == page2.tree_id and page1.lft <= page2.lft and page1.rght >= page2.rght
 
+
 @register.filter
 def is_equal_or_parent_of(page1, page2):
     """
@@ -384,10 +362,12 @@ def is_equal_or_parent_of(page1, page2):
     except AttributeError:
         return False
 
+
 # ------------------------------------------------------------------------
 def _is_sibling_of(page1, page2):
     return page1.parent_id == page2.parent_id
 
+
 @register.filter
 def is_sibling_of(page1, page2):
     """
@@ -403,6 +383,7 @@ def is_sibling_of(page1, page2):
     except AttributeError:
         return False
 
+
 # ------------------------------------------------------------------------
 @register.filter
 def siblings_along_path_to(page_list, page2):
@@ -421,6 +402,7 @@ def siblings_along_path_to(page_list, page2):
         {% endwith %}
 
     """
+
     if page_list:
         try:
             # Try to avoid hitting the database: If the current page is in_navigation,
@@ -431,7 +413,7 @@ def siblings_along_path_to(page_list, page2):
             # feincms_nav). We'll cope with the fall-out of that assumption
             # when it happens...
             ancestors = [a_page for a_page in page_list
-                                    if _is_equal_or_parent_of(a_page, page2)]
+                if _is_equal_or_parent_of(a_page, page2)]
             top_level = min((a_page.level for a_page in page_list))
 
             if not ancestors:
@@ -441,13 +423,15 @@ def siblings_along_path_to(page_list, page2):
                 p = Page(title="dummy", tree_id=-1, parent_id=None, in_navigation=False)
                 ancestors = (p,)
 
-            siblings  = [a_page for a_page in page_list
-                                if a_page.parent_id == page2.id or
-                                   a_page.level == top_level or
-                                   any((_is_sibling_of(a_page, a) for a in ancestors))]
+            siblings = [
+                a_page for a_page in page_list
+                if a_page.parent_id == page2.id
+                or a_page.level == top_level
+                or any((_is_sibling_of(a_page, a) for a in ancestors))]
+
             return siblings
-        except (AttributeError, ValueError):
-            pass
+        except (AttributeError, ValueError) as e:
+            logger.warn("siblings_along_path_to caught exception: %s", format_exception(e))
 
     return ()
 
