@@ -7,6 +7,7 @@ from django.utils.encoding import smart_str
 # backwards compatibility import
 from feincms.templatetags.fragment_tags import fragment, get_fragment, has_fragment
 
+
 register = template.Library()
 
 register.tag(fragment)
@@ -31,7 +32,7 @@ def feincms_render_region_appcontent(page, region, request):
     from feincms.content.application.models import ApplicationContent
     from feincms.templatetags.feincms_tags import _render_content
 
-    return u''.join(_render_content(content, request=request) for content in\
+    return u''.join(_render_content(content, request=request) for content in
         page.content.all_of_type(ApplicationContent) if content.region == region)
 
 
@@ -58,6 +59,7 @@ class AppReverseNode(template.Node):
         except NoReverseMatch:
             if self.asvar is None:
                 raise
+            url = ''
 
         if self.asvar:
             context[self.asvar] = url
@@ -82,7 +84,9 @@ def app_reverse(parser, token):
         {% app_reverse "mymodel_detail" "myapp.urls" name1=value1 name2=value2 %}
 
     The first argument is a path to a view. The second argument is the URLconf
-    under which this app is known to the ApplicationContent.
+    under which this app is known to the ApplicationContent. The second argument
+    may also be a request object if you want to reverse an URL belonging to the
+    current application content.
 
     Other arguments are space-separated values that will be filled in place of
     positional and keyword arguments in the URL. Don't mix positional and
